# -*- coding: utf-8 -*-

import urlparse
import requests
import hashlib
import json

from .. import constants
from ..log import logger


class API(object):

    secret = None
    base_url = None
    timeout = None

    def __init__(self, secret, base_url, timeout=None):
        self.secret = secret
        self.base_url = base_url
        self.timeout = timeout

    def create_pin(self, username, source):
        url = urlparse.urljoin(self.base_url, constants.URL_PIN_CREATE)

        data = dict(
            username=username,
            source=source,
        )

        str_data = json.dumps(data)
        sign = hashlib.md5('|'.join([self.secret, str_data])).hexdigest()

        try:
            rsp = requests.get(url, params=dict(
                data=str_data,
                sign=sign,
            ), timeout=self.timeout)

            jdata = rsp.json()

            if jdata['ret'] == 0:
                return jdata['pin']
            else:
                logger.error('rsp.ret invalid. username: %s, rsp: %s', username, jdata)
                return None

        except:
            logger.error('exc occur. username: %s', username, exc_info=True)
            return None

    def verify_pin(self, username, source, pin):
        url = urlparse.urljoin(self.base_url, constants.URL_PIN_VERIFY)

        data = dict(
            username=username,
            source=source,
            pin=pin,
        )

        str_data = json.dumps(data)
        sign = hashlib.md5('|'.join([self.secret, str_data])).hexdigest()

        try:
            rsp = requests.get(url, params=dict(
                data=str_data,
                sign=sign,
            ), timeout=self.timeout)

            jdata = rsp.json()

            if jdata['ret'] == 0:
                return True
            else:
                logger.error('rsp.ret invalid. username: %s, rsp: %s', username, jdata)
                return False

        except:
            logger.error('exc occur. username: %s', username, exc_info=True)
            return False



# -*- coding: utf-8 -*-

"""
1. 获取 corp_id, corp_secret
在pc端登录 管理后台 -> 微应用 -> 微应用设置，就可以看到

2. 生成 agent_id
在 管理后台 -> 微应用 中创建应用，就可以创建一个微应用，拿到agent_id

3. 获取要通知的 user_list/party_list
    a. 直接到网站管理后台，点击通讯录，工号即为userid；在点击左侧每个部门的时候，浏览器url中的deptId即为部门ID
    b. 调用 _get_department_list 和 _get_department_user_list 就可以获取到相关的数据，用户id即里面的userid
"""

import json
import requests


class DingBackend(object):
    """
    钉钉
    """

    SCHEMA = 'https'
    HOST = 'oapi.dingtalk.com'
    HEADERS = {
        'Content-Type': 'application/json'
    }

    # 获取token
    URL_PATH_GET_TOKEN = '/gettoken'
    # 发送消息
    URL_PATH_SEND_MESSAGE = '/message/send'
    # 获取部门列表
    URL_PATH_DEPARTMENT_LIST = '/department/list'
    # 获取部门成员列表
    URL_PATH_USER_LIST = '/user/list'

    # 最上层部门
    TOP_DEPARTMENT = 1

    def __init__(self, corp_id, corp_secret, agent_id):
        self.corp_id = corp_id
        self.corp_secret = corp_secret
        self.agent_id = agent_id

    def emit(self, title, content, user_list=None, party_list=None):
        """
        发送
        """

        full_content = '\n\n'.join([title, content])

        rsp = self._send_message(full_content, self.agent_id, user_list, party_list)

        return rsp['errcode'] == 0

    def get_user(self, userid):
        """
        获取用户信息
        :param userid:
        :return:
        """

        jdata = self._get_department_user_list(self.TOP_DEPARTMENT)

        for user in jdata['userlist']:
            if user['userid'] == userid:
                return user

        return None

    def _get_token(self):
        """
        获取access_token
        http返回格式: {"access_token":"xxxx","errcode":0,"errmsg":"ok"}
        """

        url = '%s://%s%s' % (self.SCHEMA, self.HOST, self.URL_PATH_GET_TOKEN)

        return requests.get(
            url,
            params=dict(
                corpid=self.corp_id,
                corpsecret=self.corp_secret,
            ),
            headers=self.HEADERS,
            verify=False,
        ).json()

    def _send_message(self, content, agent_id, user_list=None, party_list=None):
        """
        发送消息
        """

        access_token = self._get_token()['access_token']

        url = '%s://%s%s' % (self.SCHEMA, self.HOST, self.URL_PATH_SEND_MESSAGE)

        user_list = [str(it) for it in user_list or []]
        party_list = [str(it) for it in party_list or []]

        return requests.post(
            url,
            data=json.dumps(dict(
                touser='|'.join(user_list),
                toparty='|'.join(party_list),
                agentid=agent_id,
                msgtype='text',
                text=dict(
                    content=content
                )
            )),
            params=dict(
                access_token=access_token
            ),
            headers=self.HEADERS,
            verify=False,
        ).json()

    def _get_department_list(self):
        """
        获取部门列表
        """

        access_token = self._get_token()['access_token']

        url = '%s://%s%s' % (self.SCHEMA, self.HOST, self.URL_PATH_DEPARTMENT_LIST)

        return requests.get(
            url,
            params=dict(
                access_token=access_token
            ),
            headers=self.HEADERS,
            verify=False,
        ).json()

    def _get_department_user_list(self, department_id):
        """
        获取部门成员列表
        如果要获取顶层公司成员，好像传1就行
        """

        access_token = self._get_token()['access_token']

        url = '%s://%s%s' % (self.SCHEMA, self.HOST, self.URL_PATH_USER_LIST)

        return requests.get(
            url,
            params=dict(
                access_token=access_token,
                department_id=department_id,
            ),
            headers=self.HEADERS,
            verify=False,
        ).json()



# -*- coding: utf-8 -*-

from wtforms import StringField, PasswordField
from wtforms.validators import DataRequired
from flask_wtf import Form


class LoginForm(Form):
    """后台管理登录框"""

    username = StringField(u'帐号', validators=[DataRequired(u'该项不能为空')])
    password = PasswordField(u'密码', validators=[DataRequired(u'该项不能为空')])



# -*- coding: utf-8 -*-

import requests

from ...log import logger


class SendCloudBackend(object):
    """
    sendcloud发送邮件
    """

    SEND_URL = 'http://api.sendcloud.net/apiv2/mail/send'

    def __init__(self, api_user, api_key, sender):
        """
        初始化
        """
        self.api_user = api_user
        self.api_key = api_key
        self.sender = sender

    def emit(self, title, content, receiver_list):
        """
        发送
        """

        params = {
            'apiUser': self.api_user,
            'apiKey': self.api_key,
            'from': self.sender,
            'to': ';'.join(receiver_list),
            'subject': title,
            'plain': content,
        }

        try:
            rsp = requests.post(self.SEND_URL, params, verify=False)

            if not rsp.json()['result']:
                logger.error('send fail. status: %s, text: %s, params: %s', rsp.status_code, rsp.text, params)
                return False
            else:
                return True

        except:
            logger.error('exc occur. params: %s', params, exc_info=True)
            return False



# -*- coding: utf-8 -*-

import datetime
import re
import random

from passlib.hash import sha256_crypt
from sqlalchemy.types import TypeDecorator, TEXT
from sqlalchemy import UniqueConstraint

from extensions import db


class ListType(TypeDecorator):
    impl = TEXT

    def process_bind_param(self, value, dialect):
        return ','.join(value or [])

    def process_result_value(self, value, dialect):
        return re.split(r'\s*,\s*', value or '')


class AdminUser(db.Model):
    """
    user基类
    """

    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(255), unique=True, nullable=False)
    password = db.Column(db.String(255), nullable=False)
    create_time = db.Column(db.DateTime, nullable=False, default=datetime.datetime.utcnow)
    login_time = db.Column(db.DateTime)
    roles = db.Column(ListType)

    def __unicode__(self):
        return u'<%s %s>' % (type(self).__name__, self.username)

    def set_password(self, raw_password):
        """设置密码，要加密"""
        self.password = sha256_crypt.encrypt(raw_password)

    def check_password(self, raw_password):
        """检查密码是否合法"""
        return sha256_crypt.verify(raw_password, self.password)

    @classmethod
    def auth(cls, username, raw_password):
        """验证用户登录。如果登录成功，返回用户"""
        obj = cls.query.filter_by(username=username).first()

        if obj and obj.check_password(raw_password):
            return obj
        else:
            return None


class User(db.Model):
    """
    用户配置
    """
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(255), unique=True, nullable=False)
    ding_id = db.Column(db.String(255), nullable=False)
    mail = db.Column(db.String(255), nullable=True)
    create_time = db.Column(db.DateTime, nullable=False, default=datetime.datetime.utcnow)
    valid = db.Column(db.Boolean, default=True)

    def __unicode__(self):
        return u'<%s %s>' % (type(self).__name__, self.username)


class Pin(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)
    source = db.Column(db.String(255), nullable=False)
    code = db.Column(db.String(255), nullable=False)
    create_time = db.Column(db.DateTime, nullable=False, default=datetime.datetime.utcnow)
    expire_time = db.Column(db.DateTime)

    user = db.relationship('User', backref=db.backref('pins', order_by=db.desc(create_time)))

    __table_args__ = (
        UniqueConstraint('user_id', 'source', name='_user_source'),
    )

    @classmethod
    def create_code(cls, length):
        """
        创建code
        :return:
        """

        chars = '0123456789'
        return ''.join([random.choice(chars) for i in xrange(length)])

    def __unicode__(self):
        return u'<%s %s-%s-%s>' % (type(self).__name__, self.user_id, self.source, self.code)


class PinLog(db.Model):
    """
    操作日志
    这里都不用外键，因为关联的外键可能删除，而日志我希望保留
    """

    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(255), nullable=False)
    source = db.Column(db.String(255), nullable=False)
    pin = db.Column(db.String(255), nullable=False)
    # 所在的机器
    address = db.Column(db.String(255))
    create_time = db.Column(db.DateTime, nullable=False, default=datetime.datetime.utcnow)
    expire_time = db.Column(db.DateTime)

    def __unicode__(self):
        return u'<%s %s-%s-%s>' % (type(self).__name__, self.username, self.source, self.pin)



# -*- coding: utf-8 -*-

import functools
import datetime

from flask import redirect, url_for, flash
from flask import session, request, g, current_app
from flask import Markup
from flask_admin import AdminIndexView, BaseView, expose
from flask_admin.contrib.sqla import ModelView
from flask_admin import Admin

from ...extensions import db, admin
from ...models import AdminUser, User, Pin, PinLog
from forms import LoginForm


def register_views(app):
    """注册 admin views"""
    @app.before_request
    def inject_admin_user():
        # 注入admin_user
        g.admin_user = AdminUser.query.filter(
            AdminUser.username == session.get(current_app.config['SESSION_KEY_ADMIN_USERNAME'])).first()

    admin.add_view(AdminAuthView())

    # model
    admin.add_view(AdminUserView())
    admin.add_view(UserView())
    admin.add_view(PinView())
    admin.add_view(PinLogView())

    return admin


class AdminAuthView(BaseView):

    def is_visible(self):
        return False

    @expose('/')
    def index(self):
        return redirect(url_for('admin.index'))

    @expose('/login', methods=['GET', 'POST'])
    def login(self):
        form = LoginForm()
        if form.validate_on_submit():
            admin_user = AdminUser.auth(form.username.data, form.password.data)
            if not admin_user:
                # flash('invalid username or password')
                form.username.errors.append(u'帐号或密码错误')
                form.password.errors.append(u'帐号或密码错误')
                return self.render("admin/login.html", form=form)

            session.permanent = True
            session[current_app.config['SESSION_KEY_ADMIN_USERNAME']] = admin_user.username

            admin_user.login_time = datetime.datetime.utcnow()
            db.session.commit()

            if request.args.get('next', None):
                return redirect(request.args['next'])
            else:
                return redirect(url_for('admin.index'))

        return self.render("admin/login.html", form=form)

    @expose('/logout')
    def logout(self):
        session.pop(current_app.config['SESSION_KEY_ADMIN_USERNAME'], None)
        return redirect(url_for('admin.index'))


class AdminUserView(ModelView):
    column_exclude_list = ('password',)

    def __init__(self, *args, **kwargs):
        super(AdminUserView, self).__init__(AdminUser, db.session, *args, **kwargs)

    def is_accessible(self):
        return g.admin_user


class UserView(ModelView):

    def __init__(self, *args, **kwargs):
        super(UserView, self).__init__(User, db.session, *args, **kwargs)

    def is_accessible(self):
        return g.admin_user


class PinView(ModelView):

    column_default_sort = ('id', True)

    def __init__(self, *args, **kwargs):
        super(PinView, self).__init__(Pin, db.session, *args, **kwargs)

    def is_accessible(self):
        return g.admin_user


class PinLogView(ModelView):

    column_default_sort = ('id', True)

    def __init__(self, *args, **kwargs):
        super(PinLogView, self).__init__(PinLog, db.session, *args, **kwargs)

    def is_accessible(self):
        return g.admin_user



# -*- coding: utf-8 -*-

NAME = 'xpin'

# URL 定义

# 生成并发送PIN码
URL_PIN_CREATE = '/pin/create'

# 验证PIN码
URL_PIN_VERIFY = '/pin/verify'


# 返回码定义

# 内部错误
RET_INTERNAL = -1000

# 参数错误
RET_PARAMS_INVALID = 500

# 签名错误
RET_SIGN_INVALID = 600

# 用户无效
RET_USER_INVALID = 1000

# PIN无效
RET_PIN_VALID = 2000

# PIN过期
RET_PIN_EXPIRED = 3000


# 默认app配置
CONFIG = dict(
    # flask-sqlalchemy
    SQLALCHEMY_ECHO=False,

    # admin_user
    SESSION_KEY_ADMIN_USERNAME='admin_username',

    # PIN码长度
    PIN_LENGTH=6,

    # PIN码有效期(秒)，None代表无限
    PIN_MAX_AGE=None,

    MSG_TITLE='PIN code',
    MSG_CONTENT_TPL='{username}#{source}@{address} is applying for PIN. PIN is {pin}'
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import hashlib
import datetime
import json

from flask import Blueprint
from flask import render_template, jsonify
from flask import request, current_app, g

from ... import constants
from ...log import logger
from ..models import Pin, User, PinLog
from ..extensions import db

bp = Blueprint('frontend', __name__)


@bp.before_request
def unpack_input():
    for key in ('data', 'sign'):
        if key not in request.values:
            return jsonify(
                ret=constants.RET_PARAMS_INVALID
            )

    client_secret = current_app.config['CLIENT_SECRET']

    my_sign = hashlib.md5('|'.join(
        [client_secret, request.values['data']]
    )).hexdigest()

    if my_sign != request.values['sign']:
        return jsonify(
            ret=constants.RET_SIGN_INVALID
        )

    try:
        g.input = json.loads(request.values['data'])
    except:
        logger.error('json loads fail. request: %s', request)
        return jsonify(
            ret=constants.RET_PARAMS_INVALID
        )


@bp.route(constants.URL_PIN_CREATE)
def create_pin():

    for key in ('username', 'source'):
        if key not in g.input:
            return jsonify(
                ret=constants.RET_PARAMS_INVALID
            )

    username = g.input['username']
    source = g.input['source']

    user = User.query.filter(User.username == username).first()

    if not user or not user.valid:
        logger.error('invalid user. request: %s', request)
        return jsonify(
            ret=constants.RET_USER_INVALID,
        )

    # 判断是否是合法的企业内部用户
    if not g.ding.get_user(user.ding_id):
        logger.error('invalid ding user. request: %s', request)
        return jsonify(
            ret=constants.RET_USER_INVALID,
        )

    # 删掉旧的
    Pin.query.filter(
        Pin.user_id == user.id,
        Pin.source == source,
    ).delete()

    db.session.commit()

    pin = Pin()
    pin.user_id = user.id
    pin.source = source
    pin.code = Pin.create_code(current_app.config['PIN_LENGTH'])

    if current_app.config['PIN_MAX_AGE']:
        pin.expire_time = datetime.datetime.utcnow() + datetime.timedelta(seconds=current_app.config['PIN_MAX_AGE'])

    db.session.add(pin)
    db.session.commit()

    pin_log = PinLog()
    pin_log.username = username
    pin_log.source = source
    pin_log.pin = pin.code
    pin_log.address = request.remote_addr
    pin_log.expire_time = pin.expire_time

    db.session.add(pin_log)
    db.session.commit()

    msg_title = current_app.config['MSG_TITLE']
    msg_content = current_app.config['MSG_CONTENT_TPL'].format(
        username=username,
        source=source,
        address=request.remote_addr,
        pin=pin.code,
    )

    g.ding.emit(msg_title, msg_content, user_list=[user.ding_id])

    if g.send_cloud and user.mail:
        g.send_cloud.emit(msg_title, msg_content, [user.mail])

    return jsonify(
        ret=0,
        pin=pin.code,
    )


@bp.route(constants.URL_PIN_VERIFY)
def verify_pin():
    """
    验证pin
    :return:
    """

    for key in ('username', 'source', 'pin'):
        if key not in g.input:
            return jsonify(
                ret=constants.RET_PARAMS_INVALID
            )

    username = g.input['username']
    source = g.input['source']
    pin = g.input['pin']

    user = User.query.filter(User.username == username).first()

    if not user or not user.valid:
        logger.error('invalid user. request: %s', request)
        return jsonify(
            ret=constants.RET_USER_INVALID,
        )

    pin = Pin.query.filter(
        Pin.user_id == user.id,
        Pin.source == source,
        Pin.code == pin,
        ).first()

    if not pin:
        return jsonify(
            ret=constants.RET_PIN_VALID
        )

    if pin.expire_time and pin.expire_time < datetime.datetime.utcnow():
        return jsonify(
            ret=constants.RET_PIN_EXPIRED,
        )

    # 删掉旧的
    Pin.query.filter(
        Pin.user_id == user.id,
        Pin.source == source,
        ).delete()

    db.session.commit()

    return jsonify(
        ret=0
    )



from setuptools import setup, find_packages

setup(
    name="xpin",
    version='0.1.29',
    zip_safe=False,
    platforms='any',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    include_package_data=True,
    scripts=['xpin/bin/xpin'],
    install_requires=['requests', 'passlib',
                      'flask', 'flask_sqlalchemy', 'flask_admin', 'flask_script', 'flask_wtf'],
    url="https://github.com/dantezhu/xpin",
    license="BSD",
    author="dantezhu",
    author_email="zny2008@gmail.com",
    description="pin create/verify",
)



# -*- coding: utf-8 -*-

import os
from flask import Flask
from flask import request, g

from extensions import db, admin
import models
import views.admin

from .. import constants


def create_app(config, name=None):
    if not name:
        name = __name__

    app = Flask(name)

    app.config.from_mapping(constants.CONFIG)

    # 相对当前工作目录
    config = os.path.join(os.getcwd(), config)
    app.config.from_pyfile(config)

    configure_logging(app)
    configure_extensions(app)
    configure_context_processors(app)
    configure_handlers(app)
    configure_views(app)

    return app


def configure_logging(app):
    import logging.config

    if 'LOGGING' in app.config:
        # 有可能没配置
        app.logger and logging.config.dictConfig(app.config['LOGGING'])


def configure_extensions(app):
    """
    初始化插件
    """
    db.init_app(app)

    admin.init_app(app)


def configure_context_processors(app):
    """
    模板变量
    """
    pass


def configure_handlers(app):
    """
    before_request之类的处理
    """

    @app.before_request
    def init_libs():
        """
        初始化各种库
        """
        from .backends.ding import DingBackend
        from .backends.sendcloud import SendCloudBackend

        g.ding = DingBackend(
            app.config['DING_CORP_ID'],
            app.config['DING_CORP_SECRET'],
            app.config['DING_AGENT_ID'],
        )

        if 'SEND_CLOUD_API_USER' in app.config:
            g.send_cloud = SendCloudBackend(
                app.config['SEND_CLOUD_API_USER'],
                app.config['SEND_CLOUD_API_KEY'],
                app.config['SEND_CLOUD_SENDER'],
            )
        else:
            g.send_cloud = None


def configure_views(app):
    """
    注册views
    """
    import views.frontend

    # 注册 admin views
    views.admin.register_views(app)

    app.register_blueprint(views.frontend.bp)
